# -*- coding: utf-8 -*-
# @Time    : 2022/4/12 15:54
# @Author  : 郝岩

import hashlib
import os
import re
import sys
from traceback import format_exc

from installfunc import *

version = sys.version
if version.startswith("2"):
    python_version = "2"
    import ConfigParser
    import urllib2
    config = ConfigParser.ConfigParser()
else:
    python_version = "3"
    import configparser
    import urllib.request
    config = configparser.ConfigParser()

if os.path.exists("/etc/redhat-release"):
    os_version = "centos"
else:
    os_version = "ubuntu"



class centinit:
    def __init__(self):
        self.args = ""
        self.config = "%s/config.ini" % os.path.split(os.path.realpath(__file__))[0]
        self.urldata = {}
        self.md5data = {}
        self.version = {}
        self.pakversion = {}
        self.versiondata = {}
        self.domainsdata = {}

        self.installdir = "/usr/local/"
        self.packagedir = "/usr/local/package/"

        # 保存安装包版本检测数据
        self.checkdata = {}
        # 各地域下载ip
        self.ip = ""
        # os一致化检测软件时返回的软件列表
        self.check_result = []
        # 预加载方法
        self.initmethod()

    # 获取系统版本号
    def os_version(self):
        try:
            if os.path.exists("/etc/redhat-release"):
                version = os.popen("cat /etc/redhat-release | awk -F '.' '{print $1}' | awk '{print $NF}'")
                os_ver = "CentOS_" + version.read().strip()
            elif os.path.exists("/etc/lsb-release"):
                version = os.popen("cat /etc/lsb-release  |grep 'DISTRIB_RELEASE' |awk -F '=' '{print $2}'")
                os_ver = "Ubuntu_" + version.read().strip()
            return os_ver
        except:
            return format_exc()

    # 比对软件版本号
    def compared_version(self, ver1, ver2):
        """
        传入不带英文的版本号,特殊情况："10.12.2.6.5">"10.12.2.6"
        :param ver1: 版本号1
        :param ver2: 版本号2
        :return: ver1< = >ver2返回-1/0/1
        """
        list1 = str(ver1).split(".")
        list2 = str(ver2).split(".")

        # 循环次数为短的列表的len
        for i in range(len(list1)) if len(list1) < len(list2) else range(len(list2)):
            if int(list1[i]) == int(list2[i]):
                pass
            elif int(list1[i]) < int(list2[i]):
                return -1
            else:
                return 1
        # 循环结束，哪个列表长哪个版本号高
        if len(list1) == len(list2):
            return 0
        elif len(list1) < len(list2):
            return -1
        else:
            return 1



    # 获取主机B段地址 判断地域
    def gethostIP(self):
        try:
            # config = configparser.ConfigParser()
            config.read(self.config)
            # print(config.read(self.config))
            ip = os.popen(
                "hostname -I |xargs -n 1 |egrep  '^10\.|^192\.'|egrep -v '^10.90|^10.120|^10.255'").readlines()
            for n in ip:
                net = re.search(r"([0-9]{1,3}\.[0-9]{1,3})", n.strip('\n'))
                for data in config.items("domains"):
                    if net.group() == data[0]:
                        return net.group()
                        # pass
        except:
            print(format_exc())

    # 判断配置文件是否存在
    def checkconfig(self):
        try:
            if not os.path.exists(self.config):
                print("not found config.ini")
                sys.exit(1)
        except:
            print(format_exc())

    # 读取配置文件domains标签信息
    def getdomains(self):
        try:
            ip = self.gethostIP()
            # config = configparser.ConfigParser()
            config.read(self.config)
            for data in config.items("domains"):
                self.domainsdata[data[0]] = data[1]
            if self.domainsdata.get(ip):
                self.ip = self.domainsdata.get(ip)
            else:
                self.ip = self.domainsdata.get("default")
                os.system("   ")
        except:
            print(format_exc())

    # 读取配置文件url标签信息
    def readurl(self):
        try:
            # config = configparser.ConfigParser()
            config.read(self.config)
            if os_version == "ubuntu":
                for data in config.items("ubuntu_soft_url"):
                    self.urldata[data[0]] = data[1]
            else:
                for data in config.items("url"):
                    self.urldata[data[0]] = data[1]
        except:
            print(format_exc())

    # 读取配置文件MD5标签信息
    def readMD5(self):
        try:
            # config = configparser.ConfigParser()
            config.read(self.config)
            if os_version == "ubuntu":
                for data in config.items("ubuntu_soft_md5"):
                    self.md5data[data[0]] = data[1]
            else:
                for data in config.items("md5"):
                    self.md5data[data[0]] = data[1]
        except:
            print(format_exc())


    # 读取配置文件软件版本信息
    def readpakversion(self):
        try:
            # config = configparser.ConfigParser()
            if os.path.exists(self.packagedir + "/config.ini"):
                config.read(self.packagedir + "/config.ini")
                for data in config.items("version"):
                    self.pakversion[data[0]] = data[1]
                return self.pakversion
        except:
            print(format_exc())

    # 读取配置文件version标签信息
    def readVersion(self):
        try:
            # config = configparser.ConfigParser()
            config.read(self.config)
            if os_version == "ubuntu":
                for data in config.items("ubuntu_version"):
                    self.versiondata[data[0]] = data[1]
            else:
                for data in config.items("version"):
                    self.versiondata[data[0]] = data[1]
        except:
            print(format_exc())

    # 获取文件md5
    def getfilemd5(self, filename):
        try:
            with open(self.packagedir + filename, "r") as fd:
                return hashlib.md5(fd.read()).hexdigest()
        except:
            print(format_exc())
    
    # 获取文件version
    def getfileversion(self, filename):
        try:
            if os.path.exists(self.config):
                config.read(self.config)
                return config.get('versions', filename)
        except:
            return None

            

    # 下载指定安装包
    def getPackage(self, url):
        try:
            # url = url.replace("downloadIP",self.ip)
            url = url.replace("downloadIP", self.ip)
            print("\n正在下载:", url)
            if python_version == "3":
                fdata = urllib.request.urlopen(url)
            else:
                fdata = urllib2.urlopen(url)
            package = url.split('/')[-1]
            with open(self.packagedir + package, "wb") as code:
                code.write(fdata.read())
            if package != 'config.ini':
                return self.install(package)
        except:
            print(format_exc())

    # 下载所有安装包
    def getallPackage(self):
        try:
            for name, url in self.urldata.items():
                self.getPackage(url)
            self.getPackage(self.urldata.get('config.ini'))
        except:
            print(format_exc())

    # 检测安装包是否需要更新
    def checkpackage(self, filename, versions):
        checkdata = {}
        try:         
            fileversion = self.getfileversion(filename)
            if fileversion:
                result = self.compared_version(fileversion, versions)
                if result == 0:
                    checkdata["code"] = 0
                    checkdata["msg"] = filename
                    checkdata["value"] = ""
                else:
                    checkdata["code"] = 1
                    checkdata["msg"] = filename
                    checkdata["value"] = ""
            else:
                checkdata["code"] = 1
                checkdata["msg"] = filename
                checkdata["value"] = ""
            self.check_result.append(checkdata)
        except:
            print(format_exc())

    # 遍历所有安装包
    def checkall(self):
        try:
            self.readpakversion()
            for filename, versions in self.pakversion.items():
                self.checkpackage(filename, versions)
            return self.check_result
        except:
            print(format_exc())

    def createPackagedir(self):
        if not os.path.exists(self.packagedir):
            os.makedirs(self.packagedir)

    def initmethod(self):
        self.createPackagedir()
        self.checkconfig()
        self.readurl()
        self.getdomains()

    def init(self, packname, *args):
        self.args = args
        if packname.lower() == "all":
            self.getallPackage()
        else:
            return self.getPackage(self.urldata.get(packname))

    
    def update(self, packname, *args):
        self.args = args
        if packname.lower() == "all":
            self.getallPackage()
        else:
            self.getPackage(self.urldata.get('config.ini'))
            return self.getPackage(self.urldata.get(packname))

    # 程序解压安装逻辑
    def install(self, packagename):
        try:
            unzipdir = self.installdir + packagename.replace(".zip", "")
            if os_version == "ubuntu":
                if packagename.startswith("python2"):
                    return ubuntu_installpython2(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("flume-release"):
                    return ubuntu_installflume(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("elves"):
                    return ubuntu_installelves(self.packagedir, packagename, unzipdir)
                elif packagename.startswith("mft"):
                    return ubuntu_installmonitor4(self.packagedir, packagename, "")
                elif packagename.startswith("ipset"):
                    return ubuntu_installIpset(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("prometheus"):
                    return ubuntu_installprometheus(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("node_exporter"):
                    return ubuntu_installnode(self.packagedir, packagename, self.installdir)
                else:
                    if not os.path.exists(unzipdir):
                        os.makedirs(unzipdir)
                    os.popen("unzip -u -o  %s -d %s" % (self.packagedir + packagename, unzipdir))
            else:
                if packagename.startswith("python3"):
                    return installpython3(self.packagedir, packagename, unzipdir)
                elif packagename.startswith("python2"):
                    return installpython2(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("flume-release"):
                    return installflume(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("mft"):
                    return installmonitor4(self.packagedir, packagename, "")
                elif packagename.startswith("elves"):
                    return installelves(self.packagedir, packagename, unzipdir)
                elif packagename.startswith("ipset"):
                    return installIpset(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("prometheus"):
                    return installprometheus(self.packagedir, packagename, self.installdir)
                elif packagename.startswith("node_exporter"):
                    return installnode(self.packagedir, packagename, self.installdir)
                else:
                    if not os.path.exists(unzipdir):
                        os.makedirs(unzipdir)
                    os.popen("unzip -u -o  %s -d %s" % (self.packagedir + packagename, unzipdir))

        except:
            print(format_exc())


if __name__ == '__main__':
    # 服务安装
    if len(sys.argv) > 1:
        if sys.argv[1].lower() == "all":
            centinit().init(sys.argv[1], sys.argv[2], sys.argv[3])
        elif sys.argv[1].lower() == "check":
            centinit().checkall()
        elif sys.argv[1].lower() == "update":
            centinit().update(sys.argv[2])
    else:
        centinit().init("flume-release.zip", "gop", "test")
